#!/bin/sh
##########################################################################
#                                                                        #
#                            check_maple.sh                              #
#                                                                        #
##########################################################################
#                                                                        #
# A script for checking the version number of maple                      #
#                                                                        #
##########################################################################
#                                                                        #
# Copyright (C) 2005  Research Institute for Symbolic Computation, J.    #
# Kepler University, Linz, Austria                                       #
#                                                                        #
# Written by Christian Aistleitner                                       #
#                                                                        #
# This program is free software; you can redistribute it and/or          #
# modify it under the terms of the GNU General Public License version 2, #
# as published by the Free Software Foundation.                          #
#                                                                        #
# This program is distributed in the hope that it will be useful,        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of         #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
# GNU General Public License for more details.                           #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with this program; if not, write to the Free Software            #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,             #
# MA  02110-1301, USA.                                                   #
#                                                                        #
##########################################################################

ERROROK=0
ERRORUNDEFINEDERROR=100
ERRORWRONGUSAGE=$((ERRORUNDEFINEDERROR+1))


function print_usage() {
  cat <<USAGEOUTPUT

check_maple.sh
-------------------

usage:
  check_maple.sh MINIMALVERSION MAPLECOMMAND

with:
  MINIMALVERSION -- the minimal version of maple
                     ( e.g.: 7
                             9.5 )
  MAPLECOMMAND    -- the command with desired options
                     to start maple

return value:
  0         : maple is ok
  otherwise : problems with maple 

USAGEOUTPUT

}

	

test $# -eq 2 && \
test `echo $1 | grep -c "^[0-9]*\(\.[0-9]*\)\?"` -gt 0 && \
MINIMALMAJORVERSION=`echo $1.0 | cut -f 1 -d .` && \
test $MINIMALMAJORVERSION -ge 0 &>/dev/null && \
MINIMALMINORVERSION=`echo $1.0 | cut -f 2 -d .` && \
test $MINIMALMINORVERSION -ge 0 &>/dev/null && \
MAPLE="$2" 
if [ $? -ne 0 ]
then
  print_usage
  echo "wrong usage of check_maple.sh -- $0 $*" >&2 
  exit $ERRORWRONGUSAGE 
fi

###################################################
VERSION=`echo "kernelopts(version);" | $MAPLE 2>&1 | grep kernelopts -A 4 | grep Maple | sed -e 's/.*Maple \([0-9]*.[0-9]*\)[^0-9].*/\1/'` && \
MAJORVERSION=`echo $VERSION | cut -f 1 -d .` && \
MINORVERSION=`echo $VERSION | cut -f 2 -d .` && \
if [ $? -ne 0 -o -z "$MAJORVERSION" -o -z "$MINORVERSION" ]
then
  echo "cannot parse major or minor version of \"$MAPLE\" $VERSION" >&2
  exit $ERRORUNDEFINEDERROR  
fi

VERSIONOK=$ERROROK
if [ $MINIMALMAJORVERSION -gt $MAJORVERSION ]
then
  VERSIONOK=$ERRORUNDEFINEDERROR
elif [ $MINIMALMAJORVERSION -eq $MAJORVERSION -a $MINIMALMINORVERSION -gt $MINORVERSION ]
then
  VERSIONOK=$ERRORUNDEFINEDERROR
fi

if [ $VERSIONOK -ne $ERROROK ]
then
  echo "needed at least version $MINIMALMAJORVERSION.$MINIMALMINORVERSION of maple, but \"$MAPLE\" is only $MAJORVERSION.$MINORVERSION" >&2
fi
exit $VERSIONOK