#!/bin/sh

##########################################################################
#                                                                        #
#                      find_file_for_type.sh                             #
#                                                                        #
##########################################################################
#                                                                        #
# A script for finding the source file for a given type                  #
#                                                                        #
##########################################################################
#                                                                        #
# Copyright (C) 2005  Research Institute for Symbolic Computation, J.    #
# Kepler University, Linz, Austria                                       #
#                                                                        #
# Written by Christian Aistleitner                                       #
#                                                                        #
# This program is free software; you can redistribute it and/or          #
# modify it under the terms of the GNU General Public License version 2, #
# as published by the Free Software Foundation.                          #
#                                                                        #
# This program is distributed in the hope that it will be useful,        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of         #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
# GNU General Public License for more details.                           #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with this program; if not, write to the Free Software            #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,             #
# MA  02110-1301, USA.                                                   #
#                                                                        #
##########################################################################

ERROROK=0
ERRORUNDEFINEDERROR=100
ERRORARGUMENTS=$(($ERRORUNDEFINEDERROR+1))
ERRORTYPENOTFOUND=$(($ERRORUNDEFINEDERROR+2))
ERRORAMBIGUOUSTYPE=$(($ERRORUNDEFINEDERROR+3))

FILEFILTER=`find -follow -type f | grep -e \.as\$`


function print_usage() {
  echo
  echo "find_file_for_type.sh"
  echo "----------------------"
  echo
  echo "usage: find_file_for_type.sh TYPENAME"
  echo "  TYPENAME - the type for which to search the file"
  echo
}

if [ $# -ne 1 ]
then
  print_usage
  exit $ERRORARGUMENTS
fi

REQUESTEDTYPE=$1

## arguments are ok #################################################

SCRIPTPATH=`dirname "$0"`


COUNT=0;
for FILE in $FILEFILTER
do
  TYPE=`${SCRIPTPATH}/get_type_for_file.sh "$FILE"`
  if [ $? -eq 0 -a "$TYPE" = "$REQUESTEDTYPE" ]
  then
    SRCFILE=$FILE
    if [ $COUNT -eq 1 ]
    then
      # so type is found a second time
      exit $ERRORAMBIGUOUSTYPE
    fi
    COUNT=1
  fi
done

## checking results #################################################

if [ $COUNT -ne 1 ]
then
  exit $ERRORTYPENOTFOUND
fi

echo "$SRCFILE"
exit $ERROROK
