#!/bin/sh
##########################################################################
#                                                                        #
#                    generate_dependencies_test.sh                       #
#                                                                        #
##########################################################################
#                                                                        #
# A script for generating the dependencies of the test subdirectory      #
#                                                                        #
##########################################################################
#                                                                        #
# Copyright (C) 2005  Research Institute for Symbolic Computation, J.    #
# Kepler University, Linz, Austria                                       #
#                                                                        #
# Written by Christian Aistleitner                                       #
#                                                                        #
# This program is free software; you can redistribute it and/or          #
# modify it under the terms of the GNU General Public License version 2, #
# as published by the Free Software Foundation.                          #
#                                                                        #
# This program is distributed in the hope that it will be useful,        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of         #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
# GNU General Public License for more details.                           #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with this program; if not, write to the Free Software            #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,             #
# MA  02110-1301, USA.                                                   #
#                                                                        #
##########################################################################

ERROROK=0
ERRORUNDEFINEDERROR=100
ERRORARGUMENTS=$(($ERRORUNDEFINEDERROR+1))

function print_usage() {
cat <<HEREDOC

generate_dependencies_test.sh
-----------------------------

Usage:

  generate_dependencies_test.sh ENVIRONMENTMAKEFILE DEPENDENCYFILE

Parameters:

  ENVIRONMENTMAKEFILE - a Makefile defining properties for the aldor files
                        (e.g.: environment.mk)
  DEPENDENCYFILE      - the name of teh file to write the dependency 
                        information to


Description:

This script generates the dependencies for the $TESTDIR/$SRCDIR folder of an aldor project WITHOUT following includes.
Furthermore only one object may be defined per file. (i.e.: a file "A.as" has
 to declare "A", and nothing else.
 In addition to these restrictions, the resulting output, does not need to be
 minimal. (i.e.: A dependence situation like
    B.ao : C.as
    A.ao : B.ao2
  will lead to an output, like
    B.ao : C.as
    A.ao : B.ao C.ao
  if "C" occurs in "A.as".
HEREDOC
}

if [ $# -ne 2 ]
then
  print_usage
  exit $ERRORARGUMENTS
fi
ENVIRONMENTMK=$1
## arguments are ok #################################################
SCRIPTPATH=`dirname "$0"`
DEPENDENCYFILE=`$SCRIPTPATH/query_environment.sh DEPENDENCYFILE "$ENVIRONMENTMK"`
PROJECTROOT=`$SCRIPTPATH/query_environment.sh PROJECTROOT "$ENVIRONMENTMK"`
LIBNAME=`$SCRIPTPATH/query_environment.sh LIBNAME "$ENVIRONMENTMK"`
LIBDIR=`$SCRIPTPATH/query_environment.sh LIBDIR "$ENVIRONMENTMK"`
SRCDIR=`$SCRIPTPATH/query_environment.sh SRCDIR "$ENVIRONMENTMK"`

CWD=`pwd`
SRCFILEFILTER=`find $CWD -follow -type f | grep -e \.as\$ | grep -v \.signatures\.as\$ | grep -v ^\$CWD/tst_suit\.as | grep -v \$CWD/testcases.as`

$SCRIPTPATH/generate_dependencies.sh "$DEPENDENCYFILE" "libtestcases" "$PROJECTROOT/$LIBDIR/$SRCDIR/$DEPENDENCYFILE" "$PROJECTROOT/$LIBDIR/$LIBNAME" -- $SRCFILEFILTER >&2
LASTRESULT=$?

test $LASTRESULT -eq 0 && cat >>"$DEPENDENCYFILE" <<HEREDOC



##################################################################
#
# SIGNATUREDEPENDENCIES
#
##################################################################



HEREDOC

LASTRESULT=$?
if [ $LASTRESULT -eq 0 ]
then
  for ASFILE in $SRCFILEFILTER 
  do
    if [ $LASTRESULT -eq 0 ]
    then
      AOFILE="`echo $ASFILE | sed -e 's/\(.*\/\)\([^\/]*.\).as$/\2\.ao/'`" && \
      SIGNATURESFILE="`echo $ASFILE | sed -e 's/\.as/.signatures.as/'`" && \
cat >>"$DEPENDENCYFILE" <<HEREDOC
$AOFILE: $SIGNATURESFILE
$SIGNATURESFILE: $ASFILE
	\$(RELATIVE_ROOT)/\$(SCRIPTDIR)/mark_file_generated.sh \$(RELATIVE_ROOT)/\$(ENVIRONMENTMKFILE) \$@
	\$(RELATIVE_ROOT)/\$(SCRIPTDIR)/extract_test_signatures.sh \$< >\$@
$CWD/tst_suit.as: $SIGNATURESFILE

HEREDOC
      LASTRESULT=$?
    fi
  done
fi

cat >>"$DEPENDENCYFILE" <<HEREDOC
$CWD/tst_suit.as: \$(BUILDASFILES:%.as=%.signatures.as)
	cd $CWD && \
	\$(RELATIVE_ROOT)/scripts/generate_testsuite.sh \$(PROJECTNAME) >$CWD/tst_suit.as

tst_suit.ao: $CWD/tst_suit.as \$(RELATIVE_ROOT)/\$(INCLUDEDIR)/\$(INCLUDENAME).as \$(RELATIVE_ROOT)/\$(LIBDIR)/\$(LIBNAME)\$(LIBRARYVARIANTSUFFIX).al lib\$(TESTCASESSTUB)\$(LIBRARYVARIANTSUFFIX).al

HEREDOC

exit $LASTRESULT