#!/bin/sh

##########################################################################
#                                                                        #
#                  generate_documentation_macros.sh                      #
#                                                                        #
##########################################################################
#                                                                        #
# A script for extracting the documentation of an Aldor include file and #
# writing it to aldordoc                                                 #
#                                                                        #
##########################################################################
#                                                                        #
# Copyright (C) 2005  Research Institute for Symbolic Computation, J.    #
# Kepler University, Linz, Austria                                       #
#                                                                        #
# Written by Christian Aistleitner                                       #
#                                                                        #
# This program is free software; you can redistribute it and/or          #
# modify it under the terms of the GNU General Public License version 2, #
# as published by the Free Software Foundation.                          #
#                                                                        #
# This program is distributed in the hope that it will be useful,        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of         #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
# GNU General Public License for more details.                           #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with this program; if not, write to the Free Software            #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,             #
# MA  02110-1301, USA.                                                   #
#                                                                        #
##########################################################################

ERROROK=0
ERRORUNDEFINEDERROR=100
ERRORARGUMENTS=$(($ERRORUNDEFINEDERROR+1))

function print_usage() {
cat <<HEREDOC

generate_documentation_makros.sh
-----------------------------

Usage:

  generate_documentation_makros.sh ENVIRONMENTMAKEFILE INCLUDEFILE

Parameters:

  ENVIRONMENTMAKEFILE - a Makefile defining properties for the aldor files
                        (e.g.: environment.mk)
  INCLUDEFILE   - An Aldor source cade file (.as, or .ai)


Description:

This script generates an .tex file for INCLUDEFILE

HEREDOC
}

if [ $# -ne 2 ]
then
  print_usage
  exit $ERRORARGUMENTS
fi


ENVIRONMENTMK=$1
INCLUDEFILE=$2
## arguments are ok #################################################
SCRIPTPATH=`dirname "$0"`
PERL=`$SCRIPTPATH/query_environment.sh PERL "$ENVIRONMENTMK"`
PROJECTNAME=`$SCRIPTPATH/query_environment.sh PROJECTNAME "$ENVIRONMENTMK"`
DOCINCLUDEDOCDIR=`$SCRIPTPATH/query_environment.sh DOCINCLUDEDOCDIR "$ENVIRONMENTMK"`
NOIDCHAR="[^0-9a-zA-Z\!\?]"
NOIDCHAROREOL="\($NOTIDCHOR\|$\)"
TRUE=1
FALSE=0

#the following variable is used to work properly on the last line 
#of a file, as this line contains no final CR and therefor "read"
#already indicates a EOF.
LASTREADGAVEEOF=$FALSE

#needs $ERROR, $INCLUDEFILE
function error() {
  echo "problems parsing file $INCLUDEFILE for documentation of macros"  >&2
  echo "$ERROR"  >&2
  exit 1
}

#sets LINE
function trim_line() {
#  LINE="`echo $LINE | sed -e 's/--.*$//' -e 's/^[\ \t]*\([^\ \t]\)/\1/'`"
  LINE="`echo $LINE | sed -e 's/^[\ \t]*\([^\ \t]\)/\1/'`"
}

#sets LASTREADGAVEEOF
function check_eof() {
  if [ $? -ne 0 ]
  then
    if [ $LASTREADGAVEEOF -eq $TRUE ]
    then
      exit 0
    else
      LASTREADGAVEEOF=$TRUE
    fi
  fi 
}

#sets LINE
function myread() {
  read -r LINE
  check_eof
  trim_line
  while [ "`echo $LINE | grep -c -e \"^[\ \t]*$\|^#\"`" = "1" ]
  do
    read -r LINE
    check_eof
    trim_line
  done
}

function grab_documentation() {
  while [ "`echo $LINE | grep -c -e \"^--\"`" = "1" ] 
  do
    DOCUMENTATION="$DOCUMENTATION \n `echo $LINE | sed -e 's/.*--//' -e 's/\\\/\\\\\\\/g'`"
    myread
  done
}

exec <$INCLUDEFILE
myread


while [ true ]
do
      grab_documentation
      ISMACRO=`echo $LINE | grep -c -e ^macro[\ \t]*.*[\ \t]*==`
      if [ "$ISMACRO" = "1" ] 
      then
	MACRONAME=`echo $LINE | grep -e ^macro[\ \t]*.*[\ \t]*== | sed -e "s/^macro[\ \t]*\([^\ \t]*\)[\ \t]*\((.*==\).*/\1/" | sed -e "s/^macro[\ \t]*\([^\ \t]*\)[\ \t]*\(==\).*/\1/"`
	MACROPARAMETERS=`echo $LINE | grep -e ^macro[\ \t]*.*[\ \t]*== | sed -e "s/^macro[\ \t]*$MACRONAME[\ \t]*\((.*)\)[\ \t]*==.*/\1/"`
	if [ "$MACROPARAMETERS" = "$LINE" ]
	then
	  MACROPARAMETERS=
	fi
	if [ ! -e $DOCINCLUDEDOCDIR/tex ]
	then
	    mkdir -p $DOCINCLUDEDOCDIR/tex
	fi
	TEXFILE=$DOCINCLUDEDOCDIR/tex/$MACRONAME.tex
	echo "\clearpage" >$TEXFILE
	echo "\subsection{$MACRONAME (macro)}" >>$TEXFILE
	echo "\addefinemacro{$MACRONAME}" >>$TEXFILE
	if [ ! -z "$MACROPARAMETERS" ]
	then
	  echo "\begin{adparameters}" >>$TEXFILE
	  echo " \begin{adsnippet}" >>$TEXFILE
	  echo "  $MACROPARAMETERS" >>$TEXFILE
	  echo " \end{adsnippet}" >>$TEXFILE
	  echo "\end{adparameters}" >>$TEXFILE
	fi
	echo -e $DOCUMENTATION >>$TEXFILE
	echo "\x${PROJECTNAME}DummyCommandForSortingLongVersion{$MACRONAME                             }\input{$TEXFILE}"
      fi
      DOCUMENTATION=""
      myread
done
