#!/bin/sh

##########################################################################
#                                                                        #
#                     extract_test_signatures.sh                         #
#                                                                        #
##########################################################################
#                                                                        #
# A script for extracting the test signatures and system commands of an  #
# AldorUnit test file                                                    #
#                                                                        #
##########################################################################
#                                                                        #
# Copyright (C) 2005  Research Institute for Symbolic Computation, J.    #
# Kepler University, Linz, Austria                                       #
#                                                                        #
# Written by Christian Aistleitner                                       #
#                                                                        #
# This program is free software; you can redistribute it and/or          #
# modify it under the terms of the GNU General Public License version 2, #
# as published by the Free Software Foundation.                          #
#                                                                        #
# This program is distributed in the hope that it will be useful,        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of         #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
# GNU General Public License for more details.                           #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with this program; if not, write to the Free Software            #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,             #
# MA  02110-1301, USA.                                                   #
#                                                                        #
##########################################################################

ERROROK=0
ERRORUNDEFINEDERROR=100
ERRORARGUMENTS=$(($ERRORUNDEFINEDERROR+1))
ERRORTYPENOTFOUND=$(($ERRORUNDEFINEDERROR+2))
ERRORAMBIGUOUSTYPE=$(($ERRORUNDEFINEDERROR+3))


function print_usage() {
cat <<HEREDOC

extract_test_signatures.sh
--------------------------

Usage:

  extract_sigantures.sh ALDORSOURCEFILE


Parameters:

  ALDORSOURCEFILE - an .as file to extract the signatures from


Description:

This script extracts the signatures of tests defined in ALDORSOURCEFILE along with all #if, #else#, and #endif commands.
HEREDOC
}

if [ $# -ne 1 ]
then
  print_usage
  exit $ERRORARGUMENTS
fi
SRCFILE=$1

## arguments are ok #################################################

IFPATTERN="^#if"
ELSEPATTERN="^#else"
ENDIFPATTERN="^#endif"
TESTPATTERN="[\ \t]*\(test[a-zA-Z0-9\?\!]*\)[\ \t]*([\ \t]*)[\ \t]*:[\ \t]*([\ \t]*)[\ \t]*==.*"
grep -e "$IFPATTERN\|$ELSEPATTERN\|$ENDIFPATTERN\|$TESTPATTERN" $SRCFILE | sed -e "s/$TESTPATTERN/\1: () -> ();/"