#!/bin/sh
##########################################################################
#                                                                        #
#                     generate_dependencies_doc.sh                       #
#                                                                        #
##########################################################################
#                                                                        #
# A script for generating the dependencies of the documentation          #
# subdirectory                                                           #
#                                                                        #
##########################################################################
#                                                                        #
# Copyright (C) 2005  Research Institute for Symbolic Computation, J.    #
# Kepler University, Linz, Austria                                       #
#                                                                        #
# Written by Christian Aistleitner                                       #
#                                                                        #
# This program is free software; you can redistribute it and/or          #
# modify it under the terms of the GNU General Public License version 2, #
# as published by the Free Software Foundation.                          #
#                                                                        #
# This program is distributed in the hope that it will be useful,        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of         #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
# GNU General Public License for more details.                           #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with this program; if not, write to the Free Software            #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,             #
# MA  02110-1301, USA.                                                   #
#                                                                        #
##########################################################################

ERROROK=0
ERRORUNDEFINEDERROR=100
ERRORARGUMENTS=$(($ERRORUNDEFINEDERROR+1))
APILIBINPUTSUNSORTED=apiinputslibrary.tex.unsorted
APIINCLUDEINPUTSUNSORTED=apiinputsinclude.tex.unsorted


function print_usage() {
cat <<HEREDOC

generate_dependencies_doc.sh
-----------------------------

Usage:

  generate_dependencies_doc.sh ENVIRONMENTMAKEFILE DEPENDENCYFILE

Parameters:

  ENVIRONMENTMAKEFILE - a Makefile defining properties for the aldor files
                        (e.g.: environment.mk)
  DEPENDENCYFILE      - the name of the file to write the dependency 
                        information to


Description:

This script generates the dependencies for the extracted documentation of source code in $LIBDIR/$SRCDIR.
Furthermore it generates $APILIBINPUTSUNSORTED and $APIINCLUDEINPUTSUNSORTED
HEREDOC
}

if [ $# -ne 2 ]
then
  print_usage
  exit $ERRORARGUMENTS
fi
ENVIRONMENTMK=$1
DEPENDENCYFILE=$2
## arguments are ok #################################################
SCRIPTPATH=`dirname "$0"`
PROJECTROOT=`$SCRIPTPATH/query_environment.sh PROJECTROOT "$ENVIRONMENTMK"`
LIBDIR=`$SCRIPTPATH/query_environment.sh LIBDIR "$ENVIRONMENTMK"`
SRCDIR=`$SCRIPTPATH/query_environment.sh SRCDIR "$ENVIRONMENTMK"`
PROJECTNAME=`$SCRIPTPATH/query_environment.sh PROJECTNAME "$ENVIRONMENTMK"`
LOWERCASEPROJECTNAME=`$SCRIPTPATH/query_environment.sh LOWERCASEPROJECTNAME "$ENVIRONMENTMK"`
DOCLIBRARYDOCDIR=`$SCRIPTPATH/query_environment.sh DOCLIBRARYDOCDIR "$ENVIRONMENTMK"`


rm -f $DEPENDENCYFILE $APILIBINPUTSUNSORTED $APIINCLUDEINPUTSUNSORTED

cat >$DEPENDENCYFILE <<DEPENDENCYINFO
##################################################################
#
# DO NOT EDIT THIS FILE, IT WAS AUTOMATICALLY GENERATED VIA
#
#   $0 $*
#
# in directory: "`pwd`"                                          
#                                                                
##################################################################



##################################################################
#
# DEPENDENCE INFORMATION
#
##################################################################

DEPENDENCYINFO

#generating dependencies for library aldordoc files
for ASFILE in `find $PROJECTROOT/$LIBDIR/$SRCDIR -follow -type f  | grep -e \.as$`
do
  test $? -eq 0 && \
  TEXFILE=`      echo $ASFILE | sed -e "s|^$PROJECTROOT/$LIBDIR/$SRCDIR|$DOCLIBRARYDOCDIR|"      -e "s/\.as$/.tex/"` \
  && cat >>$DEPENDENCYFILE <<DEPENDENCYINFO
$TEXFILE: $ASFILE
	if [ ! -e "`dirname $TEXFILE`" ] ; then \$(MKDIR) -p `dirname $TEXFILE` ; fi
	\$(RELATIVE_ROOT)/\$(SCRIPTDIR)/generate_documentation_aldordoc.sh \$(RELATIVE_ROOT)/\$(ENVIRONMENTMKFILE) \$< \$@

$LOWERCASEPROJECTNAME.pdf: $TEXFILE  

DEPENDENCYINFO
  test $? -eq 0 && cat >>$APILIBINPUTSUNSORTED <<EXPORTINFO
\x${PROJECTNAME}DummyCommandForSortingLongVersion{`$SCRIPTPATH/get_type_for_file.sh $ASFILE`                             }\input{$TEXFILE}
EXPORTINFO

done

#generating dependencies for apiinputs.tex
test $? -eq 0 && cat >>$DEPENDENCYFILE <<DEPENDENCYINFO
$APIINCLUDEINPUTSUNSORTED:  \$(RELATIVE_ROOT)/\$(INCLUDEDIR)/\$(INCLUDENAME).as
	\$(RELATIVE_ROOT)/\$(SCRIPTDIR)/mark_file_generated.sh \$(RELATIVE_ROOT)/\$(ENVIRONMENTMKFILE) \$@
	\$(RELATIVE_ROOT)/\$(SCRIPTDIR)/generate_documentation_macros.sh \$(RELATIVE_ROOT)/\$(ENVIRONMENTMKFILE) $< >\$@

apiinputs.tex: $APILIBINPUTSUNSORTED $APIINCLUDEINPUTSUNSORTED
	\$(RELATIVE_ROOT)/\$(SCRIPTDIR)/mark_file_generated.sh \$(RELATIVE_ROOT)/\$(ENVIRONMENTMKFILE) \$@
	\$(ECHO) -e "\\\\\catcode\\\`\\\\\_=8\n\\\\\newcommand{\\\\\x${PROJECTNAME}DummyCommandForSortingIndex}[1]{}\n\\\\\newcommand{\\\\\x${PROJECTNAME}DummyCommandForSortingLongVersion}[1]{}" | \$(CAT) - \$^ | \$(SORT) >\$@

$LOWERCASEPROJECTNAME.pdf: apiinputs.tex

DEPENDENCYINFO


