#!/bin/sh
##########################################################################
#                                                                        #
#                       generate_dependencies_doc.sh                     #
#                                                                        #
##########################################################################
#                                                                        #
# A script for generating the dependencies of the library                #
# subdirectory                                                           #
#                                                                        #
##########################################################################
#                                                                        #
# Copyright (C) 2005  Research Institute for Symbolic Computation, J.    #
# Kepler University, Linz, Austria                                       #
#                                                                        #
# Written by Christian Aistleitner                                       #
#                                                                        #
# This program is free software; you can redistribute it and/or          #
# modify it under the terms of the GNU General Public License version 2, #
# as published by the Free Software Foundation.                          #
#                                                                        #
# This program is distributed in the hope that it will be useful,        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of         #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
# GNU General Public License for more details.                           #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with this program; if not, write to the Free Software            #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,             #
# MA  02110-1301, USA.                                                   #
#                                                                        #
##########################################################################

ERROROK=0
ERRORUNDEFINEDERROR=100
ERRORARGUMENTS=$(($ERRORUNDEFINEDERROR+1))

function print_usage() {
cat <<HEREDOC

generate_dependencies_lib.sh
----------------------------

Usage:

  generate_dependencies_lib.sh ENVIRONMENTMAKEFILE


Parameters:

  ENVIRONMENTMAKEFILE - a Makefile defining properties for the aldor files
                        (e.g.: environment.mk)

Description:

This script generates the dependencies for the $LIBDIR/$SRCDIR folder of an aldor project WITHOUT following includes.
Furthermore only one object may be defined per file. (i.e.: a file "A.as" has
 to declare "A", and nothing else.
 In addition to these restrictions, the resulting output, does not need to be
 minimal. (i.e.: A dependence situation like
    B.ao : C.as
    A.ao : B.ao2
  will lead to an output, like
    B.ao : C.as
    A.ao : B.ao C.ao
  if "C" occurs in "A.as".
HEREDOC
}

if [ $# -ne 2 ]
then
  print_usage
  exit $ERRORARGUMENTS
fi
ENVIRONMENTMK=$1
## arguments are ok #################################################
SCRIPTPATH=`dirname "$0"`
LIBNAME=`$SCRIPTPATH/query_environment.sh LIBNAME "$ENVIRONMENTMK"`
DEPENDENCYFILE=`$SCRIPTPATH/query_environment.sh DEPENDENCYFILE "$ENVIRONMENTMK"`

CWD=`pwd`
SRCFILEFILTER=`find $CWD -follow -type f | grep -e \.as\$`

$SCRIPTPATH/generate_dependencies.sh "$DEPENDENCYFILE" "$LIBNAME" -- $SRCFILEFILTER >&2
cd 