#!/bin/sh

##########################################################################
#                                                                        #
#                     generate_include_test.sh                           #
#                                                                        #
##########################################################################
#                                                                        #
# A script for generating the include file to a test library             #
#                                                                        #
##########################################################################
#                                                                        #
# Copyright (C) 2005  Research Institute for Symbolic Computation, J.    #
# Kepler University, Linz, Austria                                       #
#                                                                        #
# Written by Christian Aistleitner                                       #
#                                                                        #
# This program is free software; you can redistribute it and/or          #
# modify it under the terms of the GNU General Public License version 2, #
# as published by the Free Software Foundation.                          #
#                                                                        #
# This program is distributed in the hope that it will be useful,        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of         #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
# GNU General Public License for more details.                           #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with this program; if not, write to the Free Software            #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,             #
# MA  02110-1301, USA.                                                   #
#                                                                        #
##########################################################################

TESTCASESLIBNAME="TestCases"

ERROROK=0
ERRORUNDEFINEDERROR=100
ERRORARGUMENTS=$(($ERRORUNDEFINEDERROR+1))

function print_usage() {
cat <<HEREDOC

generate_include_test.sh
-----------------------------


Usage:

  generate_include_test.sh ENVIRONMENTMAKEFILE


Parameters:

  ENVIRONMENTMAKEFILE - a Makefile defining properties for the aldor files


Description:

This script generates an include file, used for the tests library.
HEREDOC
}



##########################################################################



if [ $# -lt 1 ]
then
  print_usage
  exit $ERRORARGUMENTS
fi

ENVIRONMENTMK="$1"
########################################

SCRIPTPATH=`dirname "$0"`
PROJECTNAME=`$SCRIPTPATH/query_environment.sh PROJECTNAME "$ENVIRONMENTMK"`
INCLUDENAME=`$SCRIPTPATH/query_environment.sh INCLUDENAME "$ENVIRONMENTMK"`
LOWERCASETESTCASESLIBNAME=`echo -n "$TESTCASESLIBNAME" | awk '{printf("%s",tolower($0))}'`

cat <<ALDORCODE
------------------------------------------------------------------------------
--
-- ${LOWERCASETESTCASESLIBNAME}.as
--
------------------------------------------------------------------------------

#assert Library${TESTCASESLIBNAME}RequiresLibrary$PROJECTNAME
#include "$INCLUDENAME"

#if GMP
#library ${LOWERCASETESTCASESLIBNAME}lib "lib${LOWERCASETESTCASESLIBNAME}-gmp.al"
#elseif DEBUG
#library ${LOWERCASETESTCASESLIBNAME}lib "lib${LOWERCASETESTCASESLIBNAME}d.al"
#else
#library ${LOWERCASETESTCASESLIBNAME}lib "lib${LOWERCASETESTCASESLIBNAME}.al"
#endif

import from ${LOWERCASETESTCASESLIBNAME}lib;
inline from ${LOWERCASETESTCASESLIBNAME}lib;
#assert Library${TESTCASESLIBNAME}IsIncluded

ALDORCODE