#!/bin/sh

##########################################################################
#                                                                        #
#                        get_type_for_file.sh                            #
#                                                                        #
##########################################################################
#                                                                        #
# A script for getting the type name of an Aldor source file             #
#                                                                        #
##########################################################################
#                                                                        #
# Copyright (C) 2005  Research Institute for Symbolic Computation, J.    #
# Kepler University, Linz, Austria                                       #
#                                                                        #
# Written by Christian Aistleitner                                       #
#                                                                        #
# This program is free software; you can redistribute it and/or          #
# modify it under the terms of the GNU General Public License version 2, #
# as published by the Free Software Foundation.                          #
#                                                                        #
# This program is distributed in the hope that it will be useful,        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of         #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
# GNU General Public License for more details.                           #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with this program; if not, write to the Free Software            #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,             #
# MA  02110-1301, USA.                                                   #
#                                                                        #
##########################################################################

ERROROK=0
ERRORUNDEFINEDERROR=100
ERRORARGUMENTS=$(($ERRORUNDEFINEDERROR+1))
ERRORTYPENOTFOUND=$(($ERRORUNDEFINEDERROR+2))

FILEFILTER=*.as

function print_usage() {
  echo
  echo "get_type_for_file.sh"
  echo "----------------------"
  echo
  echo "usage: get_type_for_file.sh FILENAME"
  echo "  FILENAME - the Aldor source file to obtain the type from"
  echo
}

if [ $# -ne 1 ]
then
  print_usage
  exit $ERRORARGUMENTS
fi

FILE=$1

if [ ! -f "$FILE" ]
then
  echo "$FILE is not a proper file"
  exit $ERRORARGUMENTS
fi

## arguments are ok #################################################

TYPE=

if [ -z "$TYPE" ]
then
  EXPR="^[\ \t]*\\thistype"
  TYPE=`grep -e "$EXPR" "$FILE" | head --lines=1 | cut -d \{  -f 2 | cut -d \} -f 1`
fi

if [ -z "$TYPE" ]
then
  #the following EXPR allows blanks at the beginning of the line. This causes troubles with included sources
  #EXPR="^[\ \t]*\(define\|extend\)\?[\ \t]*\([A-Z][a-zA-Z0-9]*\)[\ \t]*\\(:[\ \t]*\|[(].*\\).*"
  EXPR="^\(define[\ \t]*\|extend[\ \t]*\)\?\([A-Z][a-zA-Z0-9]*\)[\ \t]*\\(:[\ \t]*\|[(].*\\).*"
  TYPE=`grep -e "$EXPR" "$FILE" | head --lines=1 | sed -e "s/$EXPR/\2/"`
fi

## checking results #################################################

if [ -z "$TYPE" ]
then
  echo "cannot resolve the exported type of \"$FILE\"" >&2
  exit $ERRORTYPENOTFOUND
fi

echo "$TYPE"
exit $ERROROK
