#!/bin/sh

##########################################################################
#                                                                        #
#                       mark_file_generated.sh                           #
#                                                                        #
##########################################################################
#                                                                        #
# A script for marking files to be deleted at "make distclean"           #
#                                                                        #
##########################################################################
#                                                                        #
# Copyright (C) 2005  Research Institute for Symbolic Computation, J.    #
# Kepler University, Linz, Austria                                       #
#                                                                        #
# Written by Christian Aistleitner                                       #
#                                                                        #
# This program is free software; you can redistribute it and/or          #
# modify it under the terms of the GNU General Public License version 2, #
# as published by the Free Software Foundation.                          #
#                                                                        #
# This program is distributed in the hope that it will be useful,        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of         #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
# GNU General Public License for more details.                           #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with this program; if not, write to the Free Software            #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,             #
# MA  02110-1301, USA.                                                   #
#                                                                        #
##########################################################################

ERROROK=0
ERRORUNDEFINEDERROR=100
ERRORARGUMENTS=$(($ERRORUNDEFINEDERROR+1))


function print_usage() {
cat <<HEREDOC

mark_file_generated.sh
----------------------

Usage:

  mark_file_generated.sh ENVIRONMENTFILE FILE


Parameters:

  ENVIRONMENTMAKEFILE - a Makefile defining properties for the aldor files
  FILE                - the file (relative to the PROJECTROOT of 
                        ENVIRONMENTFILE) to be marked as being generated 
                        and therefore to be removed on distclean


Description:

This script marks FILE to be removed upon making distclean.

HEREDOC
}

if [ $# -ne 2 ]
then
  print_usage
  exit $ERRORARGUMENTS
fi
ENVIRONMENTMK=$1
FILE=$2
## arguments are ok #################################################
SCRIPTPATH=`dirname "$0"`
PROJECTROOT=`$SCRIPTPATH/query_environment.sh PROJECTROOT "$ENVIRONMENTMK"`
GENERATEDFILESFILE=`$SCRIPTPATH/query_environment.sh GENERATEDFILESFILE "$ENVIRONMENTMK"`
FILE="`dirname $FILE`/`basename $FILE`"

if [ ! -f "$PROJECTROOT/$GENERATEDFILESFILE"  ]
then
  echo "./$GENERATEDFILESFILE" >"$PROJECTROOT/$GENERATEDFILESFILE"
fi && \
if [ `grep -c -e "^$FILE$" "$PROJECTROOT/$GENERATEDFILESFILE"` -eq 0 ]
then
  echo "$FILE" | sed -e "s|$PROJECTROOT/||" >>"$PROJECTROOT/$GENERATEDFILESFILE"
fi

