#!/bin/sh

##########################################################################
#                                                                        #
#                       query_environment.sh                             #
#                                                                        #
##########################################################################
#                                                                        #
# A script for extracting the value of a variable defined at configure   #
#                                                                        #
##########################################################################
#                                                                        #
# Copyright (C) 2005  Research Institute for Symbolic Computation, J.    #
# Kepler University, Linz, Austria                                       #
#                                                                        #
# Written by Christian Aistleitner                                       #
#                                                                        #
# This program is free software; you can redistribute it and/or          #
# modify it under the terms of the GNU General Public License version 2, #
# as published by the Free Software Foundation.                          #
#                                                                        #
# This program is distributed in the hope that it will be useful,        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of         #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
# GNU General Public License for more details.                           #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with this program; if not, write to the Free Software            #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,             #
# MA  02110-1301, USA.                                                   #
#                                                                        #
##########################################################################

ERROROK=0
ERRORUNDEFINEDERROR=100
ERRORARGUMENTS=$(($ERRORUNDEFINEDERROR+1))

function print_usage() {
cat <<HEREDOC

query_environment.sh
-----------------------------

Usage:

  query_environment.sh VARIABLENAME ENVIRONMENTMAKEFILE

Parameters:

  VARIABLENAME        - the name of the variable to be extracted from
                        the ENVIRONMENTMAKEFILE
                        (e.g.: LIBNAME)
  ENVIRONMENTMAKEFILE - the environment makefile to query


Description:

This script extracts the meaning of VARIABLENAME in ENVIRONMENTMAKEFILE
HEREDOC
}

if [ $# -ne 2 ]
then
  print_usage
  exit $ERRORARGUMENTS
fi

VARIABLENAME=$1
ENVIRONMENTMK=$2
## arguments are ok #################################################

SCRIPTPATH=`dirname "$0"`


grep ^${VARIABLENAME}= "$ENVIRONMENTMK" | tail -n 1 | sed -e "s/^${VARIABLENAME}=//"
