;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This file provides an Aldor mode for EMACS.
;; Starting date: 18-Jun-2004
;; Last edited: 20-Dec-2006
;; Author: Ralf Hemmecke <ralf@hemmecke.de>
;; http://www.hemmecke.de
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Since the original file is written using a literate programming 
;; style through the noweb facility of Norman Ramsey, documentation
;; of the code can be found in aldor.el.nw.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(provide 'aldor)
(require 'comint)
(defvar aldor-path (substitute-in-file-name "$ALDORROOT/bin/aldor"))
(defvar aldor-loop-switch "-gloop")
(defun aldor-loop ()
  "Run Aldor interpreter loop in a buffer."
  (interactive)
  (if (= (count-windows) 1)
      (split-window-vertically)
    (other-window 1))
  (switch-to-buffer 
   (make-comint "Aldor" aldor-path nil aldor-loop-switch))
  (setq comint-prompt-regexp "^[^>]*[>]+ *"))
(defun aldor-send-to-loop ()
  "Send one line of input to Aldor process."
  (interactive)
  (let* ((s (current-buffer))                  ; current buffer
         (bol (progn (beginning-of-line) (point)))
         (eol (progn (end-of-line) (point)))
         (cmd (buffer-substring bol eol)))     ; current line
    ; forward cursor one line in this buffer
    (forward-line)
    ; copy line to Aldor buffer
    (set-buffer "*Aldor*")
    (goto-char (point-max))
    (insert cmd)
    (goto-char (point-max))
    ; send input to Aldor interpreter
    (comint-send-input)
    (goto-char (point-max))
    (set-window-point (get-buffer-window "*Aldor*") (point))))
(defvar aldor-mode-syntax-table nil
  "Syntax table in use in aldor-mode buffers.")

(if aldor-mode-syntax-table
    ()
  (let ((table (make-syntax-table)))
    (modify-syntax-entry ?- ". 12"   table)
    (modify-syntax-entry ?\r ">  "  table)
    (modify-syntax-entry ?\n ">  "  table)
    (modify-syntax-entry ?\t "    "  table)
    (modify-syntax-entry ?\\ ".   "  table)
    (modify-syntax-entry ?+  ".   "  table)
    (modify-syntax-entry ?*  ".   "  table)
    (modify-syntax-entry ?/  ".   "  table)
    (modify-syntax-entry ?=  ".   "  table)
    (modify-syntax-entry ?<  ".   "  table)
    (modify-syntax-entry ?>  ".   "  table)
    (modify-syntax-entry ?#  ".   "  table)
    (modify-syntax-entry ?$  ".   "  table)
    (modify-syntax-entry ?@  ".   "  table)
    (modify-syntax-entry ?_  "\\  "  table)
    (modify-syntax-entry ?%  "w   "  table)
    (modify-syntax-entry ?!  "w   "  table)
    (modify-syntax-entry ??  "w   "  table)
    (modify-syntax-entry ?\" "\"  "  table)
    (modify-syntax-entry ?\{ "(}  "  table)
    (modify-syntax-entry ?\[ "(]  "  table)
    (modify-syntax-entry ?\( "()  "  table)
    (modify-syntax-entry ?\} "){  "  table)
    (modify-syntax-entry ?\] ")[  "  table)
    (modify-syntax-entry ?\) ")(  "  table)
    (setq aldor-mode-syntax-table table)))
(defface aldor-font-lock-throw-face
  '((((class color) (background dark)) (:foreground "red"))
    (((class color) (background light)) (:foreground "red"))
    (((class grayscale) (background light)) (:foreground "DimGray" :italic t))
    (((class grayscale) (background dark)) (:foreground "LightGray" :italic t))
    (t (:bold t)))
  "Face for the Aldor throw keyword")
(defvar aldor-font-lock-throw-face 'aldor-font-lock-throw-face
  "Face for the Aldor throw keyword")

(defface aldor-font-lock-pretend-face
  '((((class color) (background dark)) (:foreground "red"))
    (((class color) (background light)) (:foreground "red"))
    (((class grayscale) (background light)) (:foreground "DimGray" :italic t))
    (((class grayscale) (background dark)) (:foreground "LightGray" :italic t))
    (t (:bold t)))
  "Face for the Aldor pretend keyword")
(defvar aldor-font-lock-pretend-face 'aldor-font-lock-pretend-face
  "Face for the Aldor pretend keyword")

(defface aldor-font-lock-assert-face
  '((((class color) (background dark)) (:foreground "red"))
    (((class color) (background light)) (:foreground "red"))
    (((class grayscale) (background light)) (:foreground "DimGray" :italic t))
    (((class grayscale) (background dark)) (:foreground "LightGray" :italic t))
    (t (:bold t)))
  "Face for the Aldor assert keyword")
(defvar aldor-font-lock-assert-face 'aldor-font-lock-assert-face
  "Face for the Aldor assert keyword")

(defface aldor-font-lock-assert-argument-face
  '((((class color) (background dark)) (:background "red"))
    (((class color) (background light)) (:background "red"))
    (((class grayscale) (background light)) (:foreground "DimGray" :italic t))
    (((class grayscale) (background dark)) (:foreground "LightGray" :italic t))
    (t (:bold t)))
  "Face for the Aldor assert arguments")
(defvar aldor-font-lock-assert-argument-face
  'aldor-font-lock-assert-argument-face
  "Face for the Aldor assert arguments")

(defface aldor-font-lock-doc-face
  '((((class color) (background dark)) (:foreground "#a0a0ff"))
    (((class color) (background light)) (:foreground "#a0a0ff"))
    (((class grayscale) (background light)) (:foreground "DimGray" :italic t))
    (((class grayscale) (background dark)) (:foreground "LightGray" :italic t))
    (t (:bold t)))
  "Face for the Aldor ++ comments")
(defvar aldor-font-lock-doc-face
  'aldor-font-lock-doc-face
  "Face for the Aldor ++ comments")

(defface aldor-font-lock-preprocessor-face
  '((((class color) (background dark)) (:foreground "MediumAquamarine"))
    (((class color) (background light)) (:foreground "MediumAquamarine"))
    (((class grayscale) (background light)) (:foreground "DimGray" :italic t))
    (((class grayscale) (background dark)) (:foreground "LightGray" :italic t))
    (t (:bold t)))
  "Face for the Aldor #if, #assert, #else etc.")
(defvar aldor-font-lock-preprocessor-face
  'aldor-font-lock-preprocessor-face
  "Face for the Aldor #if, #assert, #else etc.")

(defface aldor-font-lock-aldorunit-assert-face
  '((((class color) (background dark)) (:foreground "red"))
    (((class color) (background light)) (:foreground "red"))
    (((class grayscale) (background light)) (:foreground "DimGray" :italic t))
    (((class grayscale) (background dark)) (:foreground "LightGray" :italic t))
    (t (:bold t)))
  "Face for the AldorUnit assert... functions.")
(defvar aldor-font-lock-aldorunit-assert-face
  'aldor-font-lock-aldorunit-assert-face
  "Face for the AldorUnit assert... functions.")
(defconst id-rest "\\(\\sw\\|_.\\)*")
(defconst id-fun (concat "\\(\\<[a-z]" id-rest "\\>\\)"))
(defconst id-dom (concat "\\(\\<[A-Z]" id-rest "\\>\\)"))
(defconst aldor-font-lock-keywords
  (list
   '("\\([+][+].*\\)"
     (1 aldor-font-lock-doc-face t t))
   (list (concat "^\\(#\\)[ \t]*\\<\\("
           (mapconcat 'identity '(
              "include"
              "reinclude"
              "assert"
              "unassert"
              "if"
              "elseif"
              "else"
              "endif"
              "line"
              "error"
   ;          "pile"    -- I don't like it.
   ;          "endpile" -- I don't like it.
              "library"
              "includeDir"
              "libraryDir"
              "int"
              "quit"
           )  "\\|")
           "\\)\\>[ \t]*\\(\\sw*\\)")
     '(1 aldor-font-lock-preprocessor-face)
     '(2 aldor-font-lock-preprocessor-face)
     '(3 font-lock-variable-name-face)
   )
   '("\\(\\<from\\|to\\>\\)[ \t]+\\([^;]*\\);"
     (1 font-lock-keyword-face)
     (2 font-lock-variable-name-face))
   '("\\<\\(assert\\)\\>\\(.*\\)"
     (1 aldor-font-lock-assert-face)
     (2 aldor-font-lock-assert-argument-face))
   '("\\<\\(throw\\)\\>"
     (1 aldor-font-lock-throw-face))
   '("\\<\\(pretend\\)\\>"
     (1 aldor-font-lock-pretend-face))
   '("\\([][{}()]\\)"
     (1 font-lock-keyword-face))
   '("\\([+]->\\)"
     (1 font-lock-keyword-face))
   (list (concat "^\\(define\\|extend\\)[ \t]+" id-dom)
     '(1 font-lock-keyword-face)
     '(2 font-lock-type-face))
   (list (concat "^" id-dom)
     '(1 font-lock-type-face))     
   (list (concat "^[ \t]*\\(local\\)[ \t]+" id-fun ".*\\(==\\|($\\)")
     '(1 font-lock-keyword-face)
     '(2 font-lock-function-name-face))
   (list (concat "^[ \t]*" id-fun ".*\\(==\\|($\\)")
     '(1 font-lock-function-name-face))
   (cons (concat "\\<\\(" (regexp-opt '(
       "add"      "and"       "always"    
   ;   "assert"
       "break"    "but"      "catch"     "default"
   ;   "define"   -- Handled in "define CATEGORY"
   ;   "delay"    -- reserved Aldor keyword
       "do"       "else"     "except"    "export"
   ;   "extend"   -- Handled in "extend DOMAIN"
   ;   "fix"      -- reserved Aldor keyword
       "for"      "fluid"    "free"
   ;   "from"     -- handled in [[from and to statements]]
       "generate"
   ;   "goto"     -- THIS SHOULD NOT APPEAR IN PROGRAMS!!!
       "has"      "if"       "import"    "in"     "inline"
   ;   "is"       "isnt"      -- reserved Aldor keywords
       "iterate"
   ;   "let"      -- reserved Aldor keyword
       "local"    "macro"    "never"     "not"    "of"    "or"
   ;   "pretend"  -- Handled separately
       "ref"      "repeat"  "return"
   ;   "rule"     -- reserved Aldor keyword
       "select"    "then"
   ;   "throw"    -- Handled separately
   ;   "to"       -- handled in [[from and to statements]]
       "try"      "where"    "while"     "with"   "yield"))  "\\)\\>")
     '(1 font-lock-keyword-face))
   '("\\<\\(assert\\(True\\|False\\|Equals\\|NotEquals\\|Failed\\)\\)\\>"
     (1 aldor-font-lock-aldorunit-assert-face))
  )
  "Subdued level highlighting for Aldor modes."
)
(defun aldor-newline ()
  "Newline for Aldor major mode."
  (interactive)
  (newline)
  (aldor-indent-new-line))
(defvar aldor-mode-map
 (let ((aldor-mode-map (make-keymap)))
  (define-key aldor-mode-map [return]           'aldor-newline)
  (define-key aldor-mode-map [(shift return)]   'aldor-send-to-loop)
  (define-key aldor-mode-map [(control return)] 'aldor-loop)
  aldor-mode-map)
 "Keymap for Aldor major mode.")
(defvar aldor-offset-pair-alist
  '(
    (("("   . ")"  ) . 4)
    (("\\[" . "\\]") . 4)
    (("{"   . "}"  ) . 8)
    )
  "Regular expressions together with corresponding offset amount.")
(defvar aldor-special-line-offset-alist
  '(
    ("^[+][+][+]+"       . -1000)   ;predoc in col 0
    ("^[ \t]+[+][+][+]+" . 0)       ;indented predoc
    ("^[ \t]+[+][+]+"    . +8)      ;indented postdoc
    ("^--+"              . -1000)   ;comment in col 0
    ("^[ \t]+--+"        . 0)       ;indented comment
    ("^#"                . -1000)   ;aldor system commands
    ("^[ \t]*$"          . -1000)   ;empty line
    )
  "Regular expressions for comments together with the offsets.")
(defvar aldor-offset-alist
  (let* ((alist nil) (q nil) (p (reverse aldor-offset-pair-alist)))
    (while p
      (setq q (car p)) 
      (setq p (cdr p))
      (setq alist (cons (cons (cdar q) (- (cdr q))) alist))
      (setq alist (cons (cons (caar q) (cdr q))     alist)))
    alist))
(defun aldor-looking-at-special-line ()
  (let* (
         (return-value nil)
         (special-line-offsets aldor-special-line-offset-alist)
         )
    (while special-line-offsets
      (if (looking-at (caar special-line-offsets))
          (progn 
            (setq return-value (car special-line-offsets))
            (setq special-line-offsets nil))
        (setq special-line-offsets (cdr special-line-offsets))))
    return-value))
(defun aldor-looking-at-closing-brace ()
  (let* (
         (return-value nil)
         (p nil)
         (offsets aldor-offset-pair-alist))
    (while offsets
      (setq p (car offsets))
      (if (looking-at (concat "^[ \t]*" (cdar p)))
          (progn 
            (setq return-value (cons (cdar p) (- (cdr p))))
            (setq offsets nil))
        (setq offsets (cdr offsets))))
    return-value))
(defun aldor-looking-at-special-or-closing ()
  (let* (return-value)
    (setq return-value (aldor-looking-at-special-line))
    (if (not return-value)
        (setq return-value (aldor-looking-at-closing-brace)))
    return-value))
(defun aldor-forward-char ()
  (if (looking-at "_") (forward-char))
  (if (not (eolp))     (forward-char)))
(defun aldor-ignore-string ()
  (if (looking-at "\"") 
      (progn
        (forward-char)
        (while (and (not (eolp)) (not (looking-at "\"")))
          (aldor-forward-char)))))
(defun aldor-compute-next-indentation ()
  (let* (indent offsets forward)
    ; need to adjust the current indentation if the line starts with a
    ; closing brace.
    (setq indent (current-indentation))
    (setq offsets (aldor-looking-at-closing-brace))
    (if offsets (setq indent (- indent (cdr offsets))))

    (if (not (aldor-looking-at-special-line)) ;we might be at line 0
        ;; scan the line and adjust indent
        (progn 
          (while (not (eolp))
            (aldor-ignore-string)
            (setq offsets aldor-offset-alist)
            (setq forward nil)
            (while offsets
              (if (looking-at (caar offsets))
                  (progn 
                    (setq forward (- (length (match-data)) 1))
                    (setq indent (+ indent (cdar offsets)))
                    (setq offsets nil))
                (setq offsets (cdr offsets))))
            (if forward
                (forward-char forward)
              (aldor-forward-char)))))
    indent))
(defun aldor-skip-special-lines-backward ()
  (forward-line -1)
  (while (and (not (bobp)) (aldor-looking-at-special-line))
    (forward-line -1)))
(defun aldor-indent-line ()
  "Indentation for Aldor major mode."
  (let* (indent i)
    (beginning-of-line)
    (if (> (point) 1)
        (progn 
          (save-excursion
            (aldor-skip-special-lines-backward)
            (setq indent (aldor-compute-next-indentation)))
          ; We have the basis indentation.

          ; Now it might happen that we need to indent a
          ; special line or a line that starts with a closing
          ; paren (after initial white space).
          (setq i (aldor-looking-at-special-or-closing))
          (if i (setq indent (+ indent (cdr i))))
          (if (> 0 indent) (setq indent 0))
          (indent-line-to indent)))))
(defun aldor-indent-new-line ()
  "Indentation after pressing RET for Aldor major mode."
  (let* (indent i)
    (save-excursion
      (aldor-skip-special-lines-backward)
      (setq indent (aldor-compute-next-indentation)))
    (if (> 0 indent) (setq indent 0))
    (indent-line-to indent)))
;; hook to be run for Aldor mode 
(defvar aldor-mode-hook nil)

;; start Aldor mode
(defun aldor-mode ()
  "This is a mode intended to support program development in Aldor."
  (interactive)
  (kill-all-local-variables)
  ;; define own keys
  (use-local-map aldor-mode-map)
  (set-syntax-table aldor-mode-syntax-table)
  ;; indentation
  (set (make-local-variable 'indent-line-function) 'aldor-indent-line)
  ;; font-lock
  (set (make-local-variable 'font-lock-defaults)
       '(aldor-font-lock-keywords))
  ;; set names and run hooks
  (setq major-mode 'aldor-mode mode-name "Aldor")
  (run-hooks 'aldor-mode-hook)
)
